from System.Windows.Media import TranslateTransform
from System import Math, Random

from playareaitem import PlayAreaItem

r = Random()

class Ball(PlayAreaItem):
    def __init__(self, play_area, ball):
        self.play_area = play_area
        self.ui_element = ball
        
        self.reset()
                
    def reset(self):
        self.ui_element.RenderTransform = TranslateTransform()

        self.dx = 3 + r.NextDouble() * 8 # 0.1 # r.NextDouble() * 1
        self.dy = 1 + r.NextDouble() * 8 # 0 # r.NextDouble() * 1
        
        if r.NextDouble() < 0.5:
            self.dx = -self.dx
        if r.NextDouble() < 0.5:
            self.dy = -self.dy
    
    def _get_angle(self):
        if self.dy > 0:
            return Math.Atan(self.dx / (-self.dy)) + Math.PI
        else:
            return Math.Atan(self.dx / (-self.dy))
    angle = property(_get_angle)
    
    def _get_degree_angle(self):
        return self.angle * 180 / Math.PI
    degree_angle = property(_get_degree_angle)
    
    def _get_velocity(self):
        return (self.dx ** 2 + self.dy ** 2) ** 0.5;
    def _set_velocity(self, v):
        # XXX doesn't work yet. The idea is that the velocity would be
        # increased every time the ball hits a paddle
        angle = self.angle
        
        self.dx = ((v ** 2) / (1 + (Math.Tan(angle)**2))) ** 0.5
        self.dy = - ((v ** 2) - (self.dx ** 2)) ** 0.5
        
#        assert angle == self.angle
    velocity = property(_get_velocity, _set_velocity)
    
    def x_bounce(self):
        self.dx = -self.dx

    def move(self):
        self.ui_element.RenderTransform.X += self.dx
        self.ui_element.RenderTransform.Y += self.dy
            
        maxy = (self.play_area.ActualHeight - self.ui_element.ActualHeight) / 2
        maxx = (self.play_area.ActualWidth - self.ui_element.ActualWidth) / 2

        if self.ui_element.RenderTransform.Y < -maxy:
            self.ui_element.RenderTransform.Y = -maxy
            self.dy = -self.dy
        elif self.ui_element.RenderTransform.Y > maxy:
            self.ui_element.RenderTransform.Y = maxy
            self.dy = -self.dy
            
        if self.ui_element.RenderTransform.X < -maxx:
            self.ui_element.RenderTransform.X = -maxx
            self.dx = -self.dx
        elif self.ui_element.RenderTransform.X > maxx:
            self.ui_element.RenderTransform.X = maxx
            self.dx = -self.dx
